import java.util.*;
import java.io.*;

public class Main {

	String[] words = null;
	int linebreak = 72;

	public static void main(String args[]) {
		Main m = new Main(args);
	}

	/** Konstruktor
	 *  @param args First the input-file name and after that the position
	 *              of the linebreak.
	 */
	public Main(String[] args) {
		if (args.length < 2) {
			System.err.println("usage: java Main <input-File> <words per line>");
			System.exit(-1);
		}

		String text = "";

		try {
			File input = new File(args[0]);
			text = read(input);
		} catch (IOException exc) {
			exc.printStackTrace();
			System.exit(-2);
		}

		try {
			linebreak = Integer.parseInt(args[1]);
		} catch(NumberFormatException exc) {
			exc.printStackTrace();
			System.exit(-3);
		}


		StringTokenizer tok = new StringTokenizer(text);
		// Count of words in text.
		int wordCount = tok.countTokens();
		// array of the length of each word.
		int[] l = new int[wordCount];
		// array of words... index is the same as in <code>l</code>
		words = new String[wordCount];

		for (int i=0; i<wordCount; i++) {
			words[i] = tok.nextToken();
			l[i] = words[i].length()+1;
			System.err.println("Word ["+i+"] = \""+words[i]+"\" Length = "+l[i]);
		}

		// call the optimizer.
		long t = System.currentTimeMillis();
		System.out.println("time = " + t);
		int optimum = WC03.opt(l, linebreak);
		System.out.println("took = " + (System.currentTimeMillis() -t));

		System.out.println("\nopt = "+optimum+"\n");

		for (int col=0; col<linebreak; col++) System.out.print(".");
		System.out.println();
		for (int col=0; col<l.length; col++) {
//			System.err.println("l["+col+"] = " + l[col]);
			System.out.print(words[col]+" ");
			if (l[col] == 1) { System.out.println(); }
		}
		System.out.println();


	}

	/** Returns the File as one big String.
	 *  @param source The input file.
	 *  @return Returns the file as a big String.
	 *  @throws IOException if there are Problems reading the file.
	 */
	public static String read(File source) throws IOException {
		StringBuffer rv = new StringBuffer();
		BufferedReader in = new BufferedReader(
								new InputStreamReader(
									new FileInputStream(source)));
		if (in.ready()) {
			String text= "";
			while( (text = in.readLine()) != null) {
				rv.append(text+"\n");
			}
		}
		in.close();
		return rv.toString();
	}

    /** Writes a text to the spezified File, if allowed.
    * @param dest Destination-file to write the text in.
    * @param text Text to put in the file.
    * @param overwrite Flag if allowed to overwrite an existing file (<tt>true</tt> if alowed, <tt>false</tt> otherwise).
    * @return true if writing was succesfull, false if file aready exists and it is not allowed to overwrite it.
    * @throw IOException - if something went wrong with the writing, or creating of the destination-file.
    */
    public static boolean write(File dest, String text, boolean overwrite)
    throws IOException {
        BufferedWriter out= new BufferedWriter( new OutputStreamWriter( new FileOutputStream(dest)));
        if (overwrite) {
            out.write(text);
            out.close();
            return true;
        } else {
            return false;
        }
    }


}